//*****************************************************************************
//      Header file for:
//		3D orbital representations (using interpolation) with
//		1D plots thru maximum of wavefunction
//
//      Written by Stuart Prescott:
//              Written:                12/10/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      Other Object Classes used:
//              template arrays
//              3D points / vectors / matrices
//              graphs
//              main program data structure
//              orbital id and name
//
//*****************************************************************************

#ifndef ORBITAL_REPRESENTATION
#define ORBITAL_REPRESENTATION 1

extern "C" {
    #include <libsx.h>
}

#include "arraytem.h"
#include "vectors.h"
#include "graph.h"
#include "orbid.h"

#define pi      3.14159265
#define rtpi    1.772453851
#define rt2     1.414213562

#define ml      6		// the maximum principle quantum num for which
				// calculations will be performed
#define mrr     180		// number of points on the 1D plots

#ifndef ACTIONTYPE
#define ACTIONTYPE 1
    enum actionflagtype {WAITING_FOR_SELECTION, DRAWING_ORBITAL_DENSITY};
#endif


// the class to run the orbitals
class OrbitalView
{
    private:
	// graphics details  . . .
        Widget draw_area;			//where to put the pic
        Widget title_area;			//where to put the name

        int xpix, ypix,                 	//size of draw_area (800x600)
             x0, y0;                    	//centre of 3D drawing

        XFont font;				//to hold an unknown font
	int fonth;
	
        intarray2 *pixset;			//-1 for uncoloured pixels
						//the colourmap entry otherwise

	// picture details . . .
        class OrbitalDescriptor id;		//which orbital to draw		
        int itype;				//WF==1; DENSITY==2
        class Graph g[3];
        matrix t;				//transformation matrix
        double wfmax, wfmax0;			//value of surface

	//interpolation parameters . . .
	int mcell2,			
	    mcell, 			//== 2 * mcell2 + 1 (default==5)
	    mr;				//== 0.5 * Num of cells (default==17)

        //variables for the wfval selection process
        double lastwfval;
        int currentgraph;

	//remaining variables as data storage for plots / graphs
        double rcut, rexpand, texpand, pexpand;
        double wfscale;
        doublearray1 *r1;
        doublearray1 *wfr, *wft, *wfp;
        doublearray1 *r, *theta1, *phi1;
        doublearray3 *wf;		//_large_ array of wf vals
        doublearray2 *p;		//for the angwf function 

        char lwf[16];
        char str[60];   //[24]??

        int gcol;

    public:
        actionflagtype *choosemode;


        OrbitalView(int i)
        {
             mcell2 = int (i/2);
             mcell  = 2 * mcell2 + 1;
             mr     = int(ceil( 85 / mcell ));

             r      = new doublearray1 (-mr,mr);
             wf     = new doublearray3 (-mr,mr, -mr,mr, -mr,mr);
             r1     = new doublearray1 (0, mrr);
             pixset = new intarray2    (-120, 120, -120, 120 );
             p      = new doublearray2 (0,(2*ml), 0,ml);
             wfr    = new doublearray1 (0,mrr);
             theta1 = new doublearray1 (0,mrr);
             wft    = new doublearray1 (0,mrr);
             phi1   = new doublearray1 (0,mrr);
             wfp    = new doublearray1 (0,mrr);
        }

        ~OrbitalView()
        {    delete r;   delete wf;     delete r1;  delete pixset; delete p;
             delete wfr; delete theta1; delete wft; delete phi1;   delete wfp;}

        void Show(OrbitalDescriptor* o, int i, matrix *trans,
                    int graphcol, Widget w, Widget wt, 
                    int x, int y);

        void ClearAll();
        void TitleAll();
        void Setup1d();
        void Draw1d();
        void DrawLev();
        void Clear3d();
        void Setup3d();
        void Draw3d();
        void Redraw3d();
	void dlev3d(double wfmax, int colour1, int colour2);

        double radwf(int n, int l, double r, double alfa);
        double wfm (int i0, int j, int k);
        double angwf (int l, int m, double costh, double sinth) ;
        double hwf (double x, double y, double z);


        //functions for the wfval selector
        int WhichGraph(int x, int y);
        void DrawLevPix(int y);
        void WriteSurfaceLevel(double d);

        friend void TitleRedisplay(Widget w, int width, int height, void *data);


};    //OrbitalView


// other associated functions  
//the main gateway into the rendering  
#include "main.h"
void SetupOrbitals(struct MyProgram*);  
  
//associated fns  
void drvec (double x, double y, double z, char* lab, int x0, int y0,  
            matrix t, double scale, intarray2 *pixset, int colour);  

double axmax (double wfmax);  
  
#endif
