//*****************************************************************************
//      Header file for:
//		3D points / vectors / matrices
//
//      Written by Stuart Prescott:
//              Written:                21/08/1996
//              Last Modified:          27/10/1996
//
//      System headers:
//              Streamed I/O
//              Formatted I/O
//              Maths fns
//
//*****************************************************************************

#ifndef VECTOR_CLASS_DEFINITIONS_H
#define VECTOR_CLASS_DEFINITIONS_H 1

#include <iostream.h>
#include <math.h>

class point3
{
    public:
        double x, y, z;

      // CONSTRUCTORS
        point3 ();
        point3 (const double&, const double&, const double&);
        point3 (int, int, int);
        point3 (class vector);           // to convert from vector
        point3 (XPoint);                // to convert from screen to world

      //FUNCTIONS
        // allow streamed output
        friend ostream& operator<< (ostream&, const point3&);

        // equality of points
        int operator== (const point3&);
};


class vector : public point3
{
    public:
        //inherited from point3 class definitions
        //double x, y, z;

      //CONSTRUCTORS for initialisation
        //default
        vector ();

        //by a point
        vector (point3);

        //by 3 doubles
        vector (const double&, const double&, const double&);

        //by 3 integers
        vector (int, int, int);

        //by 2 points: _PQ_
        vector (const point3&, const point3&);

        vector (XPoint);                // to convert from XPoint

      //FUNCTIONS
        //dot product
        double dot (const vector&) const;

        //cross product
        vector cross (const vector&) const;

        //magnitude (length)
        double mag() const;

        //the unit vector
        vector unit() const;



      //OVERLOADED OPERATORS:
        //vector addition
        vector operator+ (const vector&) const;
        vector& operator+= (const vector&);

        //vector subtraction
        vector operator- (const vector&) const;
        vector& operator-= (const vector&);

        //unary minus
        vector operator- () const;

        //scalar multiplication
        friend vector operator* (const double&, const vector&);
        friend vector operator* (const vector&, const double&);

        //division by scalar
        inline vector operator/ (const double&) const;

        // equality of vectors
        int operator== (const vector&) const;

        // inequality of vectors
        int operator!= (const vector&) const;

        //member access via array notation
        double operator[] (int);

        // allow streamed output
        friend ostream& operator<< (ostream& s, const vector& v);

	//matrix * vector
	friend vector operator* (const class matrix& m, const vector& v);

	//vector * matrix
	friend vector operator* (const vector& v, const class matrix& m);
};


class matrix
{
    private:
        double m[3][3];

    public:

      //CONSTRUCTORS:
        //by 3 vectors
        matrix();
	matrix(const matrix&);
        matrix (const vector&, const vector&, const vector&);
        //by 9 doubles
        matrix (const double&, const double&, const double&,
		const double&, const double&, const double&,
		const double&, const double&, const double&);

      //FUNCTIONS
        //set a row by a vector
        inline void setrow (int, const vector&);

        //set a column by a vector
        void setcol (int, const vector&);

        //get a vector from a row
        vector getrow (int) const;

        //get a vector from a column
        vector getcol (int) const;

        //member access
        double elem ( int, int);

	//transposition of matrix
	matrix trans() const;

      //OVERLOADED OPERATORS:
        //matrix addition
        matrix operator+ (const matrix&);
        matrix& operator+= (const matrix&);

        //matrix subtraction
        matrix operator- (const matrix&);
        matrix& operator-= (const matrix&);

        //scalar multiplication
        friend matrix operator* (const double&, const matrix&);
        friend matrix operator* (const matrix&, const double&);

        //matrix multiplication
        matrix operator* (const matrix&);

        //equality of matrices
        int operator== (const matrix& p );

	//inequality of matrices
	int matrix::operator!= (const matrix& p) const;

        // allow streamed output
        friend ostream& operator<< (ostream& s, matrix v);

	//matrix * vector
	friend vector operator* (const matrix& m, const vector& v);

	//vector * matrix
	friend vector operator* (const vector& v, const matrix& m);
};


/*****************************************************************************/
/*****************************************************************************/
/*****************************************************************************/

 /*
    _ALL_ the member functions for the classes:
        point3
        vector
        matrix
*/

// DEFINITIONS FOR THE POINT3 CLASS

// constructors
inline point3::point3 ()
    {
        x=0.0;
        y=0.0;
        z=0.0;
    }

inline point3::point3 (const double& a, const double& b, const double& c=0.0)
    {
        x=a;
        y=b;
        z=c;
    }

inline point3::point3 (int a, int b, int c=0)
    {
        x=double(a);
        y=double(b);
        z=double(c);
    }

inline point3::point3 (vector v)
    {
        x=v.x;
        y=v.y;
        z=v.z;
    }

inline point3::point3 (XPoint p)
    {
        x=double(p.x);
        y=double(p.y);
        z=0.0;
    }

// allow streamed output
inline ostream& operator<< (ostream& s, const point3& p)
    {
        return s << p.x << "\t" << p.y << "\t" << p.z ;
    }

// equality operator
inline int point3::operator== (const point3& p )
    {
        return ( (x==p.x) && (y==p.y) && (z==p.z) );
    }


/*****************************************************************************/
// DEFINITIONS FOR THE VECTOR CLASS

//constructors
//by a point
inline vector::vector ()
    {
        x=0.0;
        y=0.0;
        z=0.0;
    }
inline vector::vector (point3 p)
    {
        x = p.x;
        y = p.y;
        z = p.z;
    }

//by 3 doubles
inline vector::vector  (const double& a, const double& b, const double& c = 0.0)
    {
        x=a;
        y=b;
        z=c;
    }

//by 3 integers
inline vector::vector (int a, int b, int c = 0)
    {
        x=double(a);
        y=double(b);
        z=double(c);
    }

//by 2 points: _PQ_
inline vector::vector (const point3& p, const point3& q)
    {
        x = q.x - p.x;
        y = q.y - p.y;
        z = q.z - p.z;
    }

inline vector::vector (XPoint p)
    {
        x=double(p.x);
        y=double(p.y);
        z=0.0;
    }

//FUNCTIONS
//dot product
inline double vector::dot (const vector& v) const
    {
        return ( v.x * x + v.y * y + v.z * z );
    }

//cross product
inline vector vector::cross (const vector& v) const
    {
        return vector ( y * v.z - v.y * z ,
                        z * v.x - v.z * x ,
                        x * v.y - v.x * y );
    }

//magnitude (length)
inline double vector::mag() const
    {
        return sqrt( x*x + y*y + z*z );
    }

//unit vector
inline vector vector::unit() const
    {
        return  (*this) / ( this->mag() );
    }


//OVERLOADED OPERATORS:
//vector addition
inline vector vector::operator+ (const vector& u) const
    {
        return vector ( (x+u.x), (y+u.y), (z+u.z) );
    }

inline vector& vector::operator+= (const vector& v)
    {
        x += v.x;
        y += v.y;
        z += v.z;
        return *this;
    }

//vector subtraction
inline vector vector::operator- (const vector& u) const
    {
        return vector ( (x-u.x), (y-u.y), (z-u.z) );
    }

inline vector& vector::operator-= (const vector& v)
    {
        x -= v.x;
        y -= v.y;
        z -= v.z;
        return *this;
    }

//unary minus
inline vector vector::operator- () const
    {
        return vector (-x, -y, -z);
    }

//scalar multiplication
inline vector operator* (const double& s, const vector& v)
    {
        return vector (s * v.x, s * v.y, s * v.z);
    }

inline vector operator* (const vector& v, const double& s)
    {
        return vector (s * v.x, s * v.y, s * v.z);
    }

//division by scalar
inline vector vector::operator/ (const double& s) const
    {
        return vector (x / s, y / s, z / s);
    }

// equality operator
inline int vector::operator== (const vector& p) const
    {
        return ( (x==p.x) && (y==p.y) && (y==p.y) );
    }

// inequality operator
inline int vector::operator!= (const vector& p) const
    {
        return !( (x==p.x) && (y==p.y) && (z==p.z) );
    }

//member access via array notation
inline double vector::operator[] (int s)
    {
        switch (s%3)
        {
            case 0: return (z);
            case 1: return (x);
            case 2: return (y);
        }
    }

// allow streamed output
inline ostream& operator<< (ostream& s, const vector& v)
    {
        s << v.x << "\t" << v.y << "\t" << v.z;
        return s;
    }


/*****************************************************************************/
// DEFINITIONS FOR THE MATRIX CLASS

//CONSTRUCTORS:
inline matrix::matrix ()
    {
        /*setrow (1, vector(0,0,0));
        setrow (2, vector(0,0,0));
        setrow (3, vector(0,0,0));*/
	m[0][0]=0;
	m[0][1]=0;
	m[0][2]=0;
	m[1][0]=0;
	m[1][1]=0;
	m[1][2]=0;
	m[2][0]=0;
	m[2][1]=0;
	m[2][2]=0;
    }

inline matrix::matrix (const matrix& n)
    {
	int i,j;
	for (i=0; i<=2; ++i)
	    for (j=0; j<=2; ++j)
		m[i][j]=n.m[i][j];
    }
	
//by 3 vectors
inline matrix::matrix (const vector& a, const vector& b, const vector& c)
    {
        setrow (1,a);
        setrow (2,b);
        setrow (3,c);
    }

//by 9 doubles
inline matrix::matrix (const double& a, const double& b, const double& c,
		       const double& d, const double& e, const double& f,
		       const double& g, const double& h, const double& i)
    {
        m[0][0]=a;
        m[0][1]=b;
        m[0][2]=c;
        m[1][0]=d;
        m[1][1]=e;
        m[1][2]=f;
        m[2][0]=g;
        m[2][1]=h;
        m[2][2]=i;
    }

//FUNCTIONS
//set a row by a vector
inline void matrix::setrow (int row, const vector& v)
    {
        if ((row>=1) && (row<=3))
        {
            m[row-1][0] = v.x;
            m[row-1][1] = v.y;
            m[row-1][2] = v.z;

        }
        else
        {
            cerr << "Bad matrix row "
                 << row << "\n";
        }
    }

//set a column by a vector
inline void matrix::setcol (int col, const vector& v)
    {
        if ((col>=1) && (col<=3))
        {
            m[0][col-1] = v.x;
            m[1][col-1] = v.y;
            m[2][col-1] = v.z;
        }
        else
        {
            cerr << "Bad matrix column "
                 << col << "\n";
        }
    }

//get a vector from a row
inline vector matrix::getrow (int row) const
    {
        if ((row>=1) && (row<=3))
        {
                return vector ( m[row-1][0], m[row-1][1], m[row-1][2] );
        }
        else
        {
            cerr << "Bad matrix row "
                 << row << "\n";
            return vector (0,0,0);
        }
    }

//get a vector from a column
inline vector matrix::getcol (int col) const
    {
        if ((col>=1) && (col<=3))
        {
            return vector ( m[0][col-1], m[1][col-1], m[2][col-1] );
        }
        else
        {
            cerr << "Bad matrix column "
                 << col << "\n";
            return vector (0,0,0);
        }
    }

//member access
inline double matrix::elem (int r, int c)
    {
        if ((r>=1) && (r<=3) && (c>=1) && (r<=3))
        {
            return ( m[r-1][c-1] );
        }
        else
        {
            cerr << "Bad matrix co-ords "
                 << r << " " << c << "\n";
            return (0);
        }
    }

//transposition matrix
inline matrix matrix::trans() const
    {
	return matrix( m[1][1], m[2][1], m[3][1],
		       m[1][2], m[2][2], m[3][2],
		       m[1][3], m[2][3], m[3][3] );
    }

//OVERLOADED OPERATORS:
//matrix addition
inline matrix matrix::operator+ (const matrix& v)
    {
        matrix a;
        int c;
        for (int r=0; r<3; ++r)
        {
            for (c=0; c<3; ++c)
                a.m[r][c] = m[r][c] + v.m[r][c];
        }
        return a;
    }

inline matrix& matrix::operator+= (const matrix& v)
    {
        int c;
        for (int r=0; r<3; ++r)
        {
            for (c=0; c<3; ++c)
                this->m[r][c] += v.m[r][c];
        }
        return *this;
    }

//matrix subtraction
inline matrix matrix::operator- (const matrix& v)
    {
        matrix a;
        int c;
        for (int r=0; r<3; ++r)
        {
            for (c=0; c<3; ++c)
                a.m[r][c] = this->m[r][c] - v.m[r][c];
        }
        return a;
    }
inline matrix& matrix::operator-= (const matrix& v)
    {
        int c;
        for (int r=0; r<3; ++r)
        {
            for ( c=0; c<3; ++c)
                this->m[r][c] += v.m[r][c];
        }
        return *this;
    }

//scalar multiplication
inline matrix operator* (const double& s, const matrix& v)
    {
        matrix a;
        int c;
        for (int r=0; r<3; ++r)
        {
            for ( c=0; c<3; ++c)
                a.m[r][c] = s * v.m[r][c];
        }
        return a;
    }
inline matrix operator* (const matrix& v, const double& s)
    {
        matrix a;
        int c;
        for (int r=0; r<3; ++r)
        {
            for ( c=0; c<3; ++c)
                a.m[r][c] = s * v.m[r][c];
        }
        return a;
    }

//matrix multiplication
inline matrix matrix::operator* (const matrix& y)
    {   matrix a;
        //vector v;
        int c;
        for (int r=1; r<=3; ++r)
        {
            for ( c=1; c<=3; ++c)
            {
                //v = this->getrow(r);
                a.m[r-1][c-1] = ( (this->getrow(r)).dot ( y.getcol(c) ) );
            }
        }
        return a;
    }

// equality operator
inline int matrix::operator== (const matrix& p )
    {
        return ( (m[0][0]==p.m[0][0]) &&
                 (m[0][1]==p.m[0][1]) &&
                 (m[0][2]==p.m[0][2]) &&
                 (m[1][0]==p.m[1][0]) &&
                 (m[1][1]==p.m[1][1]) &&
                 (m[1][2]==p.m[1][2]) &&
                 (m[2][0]==p.m[2][0]) &&
                 (m[2][1]==p.m[2][1]) &&
                 (m[2][2]==p.m[2][2]) );
    }

// inequality operator
inline int matrix::operator!= (const matrix& p) const
    {
        return !((m[0][0]==p.m[0][0]) &&
                 (m[0][1]==p.m[0][1]) &&
                 (m[0][2]==p.m[0][2]) &&
                 (m[1][0]==p.m[1][0]) &&
                 (m[1][1]==p.m[1][1]) &&
                 (m[1][2]==p.m[1][2]) &&
                 (m[2][0]==p.m[2][0]) &&
                 (m[2][1]==p.m[2][1]) &&
                 (m[2][2]==p.m[2][2]) );
    }

// allow streamed output
inline ostream& operator<< (ostream& s, matrix v)
    {
        int c;
        for (int r=0; r<3; ++r)
        {
            for ( c=0; c<3; ++c)
                 s << v.m[r][c] << "\t";
            s << "\n";
        }
        return s;
    }

//******************************************************************************
//******************************************************************************
//FRIEND FUNCTIONS

//matrix * vector
inline vector operator* (const matrix& m, const vector& v)
    {
	return vector ( m.getrow(1).dot(v),
			m.getrow(2).dot(v),
			m.getrow(3).dot(v) );
    }

//vector * matrix
inline vector operator* (const vector& v, const matrix& m)
    {
	return vector ( v.dot(m.getcol(1)),
			v.dot(m.getcol(2)),
			v.dot(m.getcol(3)) );
    }

#endif
